/*
 * Copyright (C) 2012 by
 *   MetraLabs GmbH (MLAB), GERMANY
 * and
 *   Neuroinformatics and Cognitive Robotics Labs (NICR) at TU Ilmenau, GERMANY
 * All rights reserved.
 *
 * Contact: info@mira-project.org
 *
 * Commercial Usage:
 *   Licensees holding valid commercial licenses may use this file in
 *   accordance with the commercial license agreement provided with the
 *   software or, alternatively, in accordance with the terms contained in
 *   a written agreement between you and MLAB or NICR.
 *
 * GNU General Public License Usage:
 *   Alternatively, this file may be used under the terms of the GNU
 *   General Public License version 3.0 as published by the Free Software
 *   Foundation and appearing in the file LICENSE.GPL3 included in the
 *   packaging of this file. Please review the following information to
 *   ensure the GNU General Public License version 3.0 requirements will be
 *   met: http://www.gnu.org/copyleft/gpl.html.
 *   Alternatively you may (at your option) use any later version of the GNU
 *   General Public License if such license has been publicly approved by
 *   MLAB and NICR (or its successors, if any).
 *
 * IN NO EVENT SHALL "MLAB" OR "NICR" BE LIABLE TO ANY PARTY FOR DIRECT,
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
 * THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF "MLAB" OR
 * "NICR" HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * "MLAB" AND "NICR" SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND "MLAB" AND "NICR" HAVE NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS.
 */

/**
 * @file shared_ptr.hpp
 *    Allows to use boost::shared_ptr for serialization like a
 *    normal pointer.
 *
 * @author Erik Einhorn
 * @date   2010/07/14
 */

#ifndef _MIRA_SHARED_PTR_HPP_
#define _MIRA_SHARED_PTR_HPP_

#ifndef Q_MOC_RUN
#include <boost/shared_ptr.hpp>
#endif

#include <serialization/ReflectorInterface.h>
#include <serialization/SplitReflect.h>

#include <serialization/IsTransparentSerializable.h>
#include <serialization/IsObjectTrackable.h>

namespace mira {

////////////////////////////////////////////////////////////////////////////////

/// Specialization of the non-intrusive reflect for boost::shared_ptr.
template<typename Reflector, typename T>
void reflectRead(Reflector& r, boost::shared_ptr<T>& t)
{
	T* p = t.get();
	r.delegate(p);
}

/// Specialization of the non-intrusive reflect for boost::shared_ptr.
template<typename Reflector, typename T>
void reflectWrite(Reflector& r, boost::shared_ptr<T>& t)
{
	T* p;
	r.delegate(p);
	r.sharedPointerReset(t, p);
}

template<typename Reflector, typename T>
void reflect(Reflector& r, boost::shared_ptr<T>& t)
{
	splitReflect(r, t);
}

template<typename T, typename SerializerTag>
class IsTransparentSerializable<boost::shared_ptr<T>,SerializerTag> : public std::true_type {};

template <typename T>
class IsObjectTrackable<boost::shared_ptr<T>> : public std::false_type {};

////////////////////////////////////////////////////////////////////////////////

} // namespace

#endif
