/*
 * Copyright (C) 2012 by
 *   MetraLabs GmbH (MLAB), GERMANY
 * and
 *   Neuroinformatics and Cognitive Robotics Labs (NICR) at TU Ilmenau, GERMANY
 * All rights reserved.
 *
 * Contact: info@mira-project.org
 *
 * Commercial Usage:
 *   Licensees holding valid commercial licenses may use this file in
 *   accordance with the commercial license agreement provided with the
 *   software or, alternatively, in accordance with the terms contained in
 *   a written agreement between you and MLAB or NICR.
 *
 * GNU General Public License Usage:
 *   Alternatively, this file may be used under the terms of the GNU
 *   General Public License version 3.0 as published by the Free Software
 *   Foundation and appearing in the file LICENSE.GPL3 included in the
 *   packaging of this file. Please review the following information to
 *   ensure the GNU General Public License version 3.0 requirements will be
 *   met: http://www.gnu.org/copyleft/gpl.html.
 *   Alternatively you may (at your option) use any later version of the GNU
 *   General Public License if such license has been publicly approved by
 *   MLAB and NICR (or its successors, if any).
 *
 * IN NO EVENT SHALL "MLAB" OR "NICR" BE LIABLE TO ANY PARTY FOR DIRECT,
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
 * THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF "MLAB" OR
 * "NICR" HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * "MLAB" AND "NICR" SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND "MLAB" AND "NICR" HAVE NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS.
 */

/**
 * @file ThreadInfoView.h
 *    TODO Add description.
 *
 * @author Tim Langner
 * @date   14.01.2011
 */

#ifndef _MIRA_THREADINFOVIEW_H_
#define _MIRA_THREADINFOVIEW_H_

#include <serialization/adapters/Qt/QAction>

#include <rcp/ViewPart.h>

#include <thread/ThreadMonitor.h>

#include <QTableWidget>
#include <QTimer>

namespace mira
{

///////////////////////////////////////////////////////////////////////////////

class ThreadInfoView : public ViewPart
{
MIRA_META_OBJECT(ThreadInfoView,
                ("Category", "Information")
                ("Name", "Thread Monitor")
                ("Description", "Displays informations about threads"))

Q_OBJECT

public:
	ThreadInfoView();
	virtual ~ThreadInfoView();

	template<typename Reflector>
	void reflect(Reflector& r)
	{
		r.property("Show unknown threads",
		           getter<bool>(boost::bind(actionGetter, mShowUnknownThreads)),
		           setter<bool>(boost::bind(actionSetter, mShowUnknownThreads, _1)),
		           "", false);

		r.property("Show hidden threads",
		           getter<bool>(boost::bind(actionGetter, mShowHiddenThreads)),
		           setter<bool>(boost::bind(actionSetter, mShowHiddenThreads, _1)),
		           "", false);
	}

	virtual Qt::DockWidgetArea getDefaultDockArea() const {
		return Qt::BottomDockWidgetArea;
	}

	void updateInfo();

public slots:
	void update();

protected:
	virtual QWidget* createPartControl();

protected:

	virtual void timerEvent(QTimerEvent *e);

private:

	void updateTiming(const ThreadMonitor::ThreadInfoVector& info);

private:
	QAction* mShowUnknownThreads;
	QAction* mShowHiddenThreads;

	/// stores per thread information for timing and performance analysis
	struct ThreadStat {

		ThreadStat():
			cpuUsage(0.0f), valid(false), zombie(false) {}
		ThreadStat(const Duration& lastTime) :
			lastTotalTime(lastTime), cpuUsage(0.0f), valid(false), zombie(false) {}

		Duration lastTotalTime;

		ThreadMonitor::ThreadInfo info;
		float cpuUsage;
		bool valid; ///< true, if cpuUsage and other statistics are valid

		bool zombie; ///< used to detect threads that have passed away
	};

	/// timing information for each thread
	std::map<ThreadID,ThreadStat> mThreadStats;
	boost::mutex mInfoMutex;
	boost::thread mInfoWorkerThread;

	Time mLastTimingMeasurement;

	class UI;
	UI* ui;
};

///////////////////////////////////////////////////////////////////////////////

class ThreadInfoView::UI : public QWidget
{
	Q_OBJECT
public:
	UI(QWidget* parent);
	void setupUi();

public:
	QTableWidget* table;
};

///////////////////////////////////////////////////////////////////////////////

}

#endif
